@@ -79,6 +79,12 @@ gchar *mm_bcd_to_string (const guint8 *bcd,
                          gsize bcd_len,
                          gboolean low_nybble_first);
 
+/*
+ * Convert a string into a quoted and escaped string. Returns a new
+ * allocated string. Follows ITU V.250 5.4.2.2 "String constants".
+ */
+gchar *mm_at_quote_string (const gchar *input);
+
 /*****************************************************************************/
 /* VOICE specific helpers and utilities */
 /*****************************************************************************/
@@ -136,6 +142,7 @@ GRegex    *mm_3gpp_cgev_regex_get (void);
 GRegex    *mm_3gpp_cusd_regex_get (void);
 GRegex    *mm_3gpp_cmti_regex_get (void);
 GRegex    *mm_3gpp_cds_regex_get (void);
+GRegex    *mm_3gpp_cbm_regex_get (void);
 
 /* AT+WS46=? response parser: returns array of MMModemMode values */
 GArray *mm_3gpp_parse_ws46_test_response (const gchar  *response,
@@ -565,6 +572,15 @@ gboolean mm_sim_parse_cpol_test_response (const gchar  *response,
                                           guint        *out_max_index,
                                           GError      **error);
 
+/* Parse operator name and mnc length */
+gchar *mm_sim_convert_spn_to_utf8 (const guint8  *bin,
+                                   gsize          len,
+                                   GError       **error);
+
+guint mm_sim_validate_mnc_length (const guint8  *bin,
+                                  gsize          len,
+                                  GError       **error);
+
 /*****************************************************************************/
 
 /* Useful when clamp-ing an unsigned integer with implicit low limit set to 0,
@@ -587,4 +603,15 @@ gboolean mm_sim_parse_cpol_test_response (const gchar  *response,
 /* Helper function to decode eid read from esim */
 gchar *mm_decode_eid (const gchar *eid, gsize eid_len);
 
+typedef struct {
+    const gchar *str;
+    const guint  val;
+} MMStringUintMap;
+
+/* Helper to look up string value in a map and return corresponding guint */
+guint mm_string_uint_map_lookup (const MMStringUintMap *map,
+                                 const gsize            map_size,
+                                 const gchar           *str,
+                                 const guint            default_value);
+
 #endif  /* MM_MODEM_HELPERS_H */
